function [THETA_hat,THETA_m,G_BIAS]=MTi_ecf(MT_SINGLE,time)
% MTi_ecf
% 
% Purpose: 
%           Attitude estimation from calibrated MTi data using an explicit
%           complementary filter.
%
% Inputs:      MT_SINGLE (data from MTi, duration x 12)
%              time (time vector)
%
% Outputs:     THETA_hat (estimated attitude vector)
%              THETA_m (algebraic measured attitude)
%
% Subroutines:  
%                  
%
% Record of revisions:
% Date:        Programmer:                    Description of change:
%--------------------------------------------------------------------------
% 02 May 12    Fredrik Dukan                  org.
%
%--------------------------------------------------------------------------

%% Heading 
l_data=length(MT_SINGLE);

h=(time(8)-time(4))/4; %time step [s]

phi_m=zeros(l_data,1); %simple roll and pitch from acc:
theta_m=zeros(l_data,1);
psi_m=zeros(l_data,1); 

q_hat=zeros(4,l_data+1); 
q_hat(:,1)=euler2q(phi_m(1),theta_m(1),psi_m(1)); %IC from algebraic measurements

bg=zeros(3,l_data+1); 
bg(:,1)=[-0.001324550643655;-0.000530147120334;0.002055191891841];
w_mes=zeros(3,l_data); 

THETA_hat=zeros(3,l_data+1); 


for i=1:l_data
acc_x=MT_SINGLE(i,1);
acc_y=MT_SINGLE(i,2);
acc_z=MT_SINGLE(i,3);

p_m=MT_SINGLE(i,4);
q_m=MT_SINGLE(i,5);
r_m=MT_SINGLE(i,6);

mag_x=MT_SINGLE(i,7);
mag_y=MT_SINGLE(i,8);
mag_z=MT_SINGLE(i,9);

%% Algebraic attitude measurements from calibrated data:
%Note: R_i_m is algebraic measurement of the rotation matrix
% Roll:
phi_m(i)=atan2(acc_y,acc_z)+pi; % atan(a_y/a_z)

%modify if upside down:
if abs(phi_m(i))>pi && phi_m(i)>=0
phi_m(i)=phi_m(i)-2*pi;
elseif abs(phi_m(i))>pi && phi_m(i)<0
phi_m(i)=phi_m(i)+2*pi;  
end

%Pitch:
theta_m(i)=atan2(acc_x,sqrt(acc_y^2+acc_z^2)); % -atan(a_x/sqrt(a_y^2+a_z^2)) (+) is correct (FD) 
    
%Rotation matrices:
R_x_m=[1 0 0;
       0 cos(phi_m(i)) -sin(phi_m(i));
       0 sin(phi_m(i)) cos(phi_m(i))];    
    
R_y_m=[cos(theta_m(i)) 0 sin(theta_m(i));
       0 1 0;
       -sin(theta_m(i)) 0 cos(theta_m(i))];

h_xyz_m=R_y_m*R_x_m*[mag_x;mag_y;mag_z] ; % use magnetometer for heading, (11.225) in Fossen 11:

% Yaw:
psi_m(i)=-atan2(h_xyz_m(2),h_xyz_m(1)); % substitute for (11.229) in Fossen 11:

% R_z_m=[cos(psi_m(i)) -sin(psi_m(i)) 0;
%        sin(psi_m(i)) cos(psi_m(i)) 0;
%        0 0 1];    

% % Final rotation matrix (body to NED):
%R_nb_m=R_z_m*R_y_m*R_x_m;  

%% Explicit Complementary Filter:-------------------------------------------

% step 1: Find R_nb(q_hat); -----------------------------------------------
if i==1
q_hat(:,i)=euler2q(phi_m(1),theta_m(1),psi_m(1)); %IC from algebraic measurements
end

eta = q_hat(1,i);
eps = q_hat(2:4,i);

S = [    0   -eps(3)   eps(2)
      eps(3)     0    -eps(1)
     -eps(2)   eps(1)     0 ];

R_nb_q = eye(3) + 2*eta*S + 2*S^2;
% end step 1---------------------------------------------------------------

% step 2: Find normalized acceleration and magnetometer measurements: -----
v_1b=[acc_x;acc_y;acc_z]/sqrt(acc_x^2+acc_y^2+acc_z^2);  % (11.299)
v_2b=[mag_x;mag_y;mag_z]/sqrt(mag_x^2+mag_y^2+mag_z^2);  % (11.299)
% end step 2---------------------------------------------------------------

% step 3: Find estimate of v_ib:-------------------------------------------
v_1b_hat=R_nb_q'*[0;0;-1] ;  % (11.306) in Fossen 11 (Note change in sign to -1 and transpose.)

mag_nx=0.28; % NOTE: NED magfield from measurements while heading North with zero roll and pitch.
mag_ny=0;
mag_nz=0.96;

v_2b_hat=R_nb_q'*[mag_nx;mag_ny;mag_nz]/sqrt(mag_nx^2+mag_ny^2+mag_nz^2); % (11.308) in Fossen 11 (Note change to ned and transpose)
% end step 3---------------------------------------------------------------

%step 4: find w_mes_b:-----------------------------------------------------
k_vex=[1;1];

temp=0.5*k_vex(1)*(v_1b*v_1b_hat'-v_1b_hat*v_1b')+0.5*k_vex(2)*(v_2b*v_2b_hat'-v_2b_hat*v_2b');
w_mes(:,i)=-[temp(3,2);temp(1,3);temp(2,1)]; % w_mes=-vex(temp)
%end step 4----------------------------------------------------------------

%step 5: find gyro bias :--------------------------------------------------
k_is=1; %Start up bias gain
T_f=20; %time constant bias gain
K_if=[0.5 0 0;
      0 0.5 0;
      0 0 0.5]; %final bias gain
 
K_i=K_if*(1+(k_is/K_if(1,1)-1))*exp(-(time(i))/T_f);

bg(:,i+1)=bg(:,i)-h*0.5*K_i*w_mes(:,i);   %Euler integration of gyro bias
%end step 5----------------------------------------------------------------

%step 6: find T(q) :-------------------------------------------------------
%eta  = q(1); eps1 = q(2); eps2 = q(3); eps3 = q(4); 
  
T_q = 0.5*[...
   -eps(1) -eps(2) -eps(3)        
    eta  -eps(3)  eps(2)
    eps(3)  eta  -eps(1)
   -eps(2)  eps(1)  eta ];
%end step 6----------------------------------------------------------------

%step 7: find q:-----------------------------------------------------------
K_p=[0.3 0 0;
     0 0.3 0;
     0 0 10];

w_imu=[p_m;q_m;r_m]; % IMU gyro measurements [rads/s]
w_q=w_imu-bg(:,i+1)+K_p*w_mes(:,i);  % bi calculation

q_hat(:,i+1)=q_hat(:,i)+h*T_q*w_q;  %Euler integration 
q_hat(:,i+1)=q_hat(:,i+1)/sqrt(q_hat(:,i+1)'*q_hat(:,i+1)); %Normalize q
%end step 7----------------------------------------------------------------

%step 8: quaternions to euler angles::-------------------------------------
eta = q_hat(1,i+1);
eps = q_hat(2:4,i+1);

S = [0       -eps(3)   eps(2)
     eps(3)     0     -eps(1)
    -eps(2)   eps(1)       0 ];

R= eye(3) + 2*eta*S + 2*S^2;
 
phi_h   = atan2(R(3,2),R(3,3));
theta_h = -asin(R(3,1));
psi_h   = atan2(R(2,1),R(1,1));
%end step 8----------------------------------------------------------------

%Output estimated attitude:
THETA_hat(:,i)=[phi_h;theta_h;psi_h]; %*180/pi; 

end %Explicit Complementary Filter

THETA_hat=THETA_hat(:,1:l_data);
G_BIAS=bg(:,1:l_data);

THETA_m=[phi_m theta_m psi_m]'; %algebraic measurement attitude 

